@@ -235,6 +235,9 @@ class Storage(object):
 
 
 class Config(object):
+    def __init__(self):
+        self._data = {}
+
     def read_from_file(self, filename):
         with open(filename) as f:
             self.read_from_string(f.read())
@@ -260,17 +263,22 @@ class Config(object):
                         data[item + '_' + k] = v
 
 
-        for key, value in data.items():
-            setattr(self, key, value)
+        self._data.update(data)
 
-    def update_from_args(self, args):
+    def update_from_dict(self, args):
         """Gets settings from the Namespace object, generated by argparse."""
-        for key, value in args._get_kwargs():
+        for key, value in args.items():
             if value is not None:
                 if key in ('adapters', 'plugins'):
                     value = value.split(',')
 
-                setattr(self, key, value)
+                self._data[key] = value
+
+    def __repr__(self):
+        return 'Config: {}'.format(self._data)
+
+    def __getattr__(self, name):
+        return self._data[name]
 
 
 class Bot(object):
@@ -305,25 +313,36 @@ class Bot(object):
 
 
         config_filename = 'thebot.conf'
-        config = Config()
 
-        config.read_from_string("""
-            adapters: [console]
-            plugins: [image]
+        default_config = """
             unittest: no
-            storage_filename: thebot.storage
-            log_filename: thebot.log
-            verbose: no
-        """)
+        """
+
+        # read config first time, to figure out possible adapter
+        config = Config()
+        config.read_from_string(default_config)
+
+        parser = Bot.get_general_options()
+        defaults = dict(parser.parse_args([])._get_kwargs())
+        config.update_from_dict(defaults)
 
         if os.path.exists(config_filename):
             config.read_from_file(config_filename)
 
-        parser = Bot.get_general_options()
         args, unknown = parser.parse_known_args(
             list(filter(lambda x: x not in ('--help', '-h'), command_line_args))
         )
-        config.update_from_args(args)
+        # we override options only if there was specified value different from default
+        args = dict(
+            (key, value)
+            for key, value in args._get_kwargs()
+                if value != defaults.get(key)
+        )
+        config.update_from_dict(args)
+
+
+        # now, load plugin and adapter classes, collect their options
+        # and parse command line again
 
         if adapters is None:
             adapter_classes = map(lambda a: load(a, 'Adapter'), config.adapters)
@@ -343,8 +362,28 @@ class Bot(object):
             if hasattr(cls, 'get_options'):
                 cls.get_options(parser)
 
+
+        # now, reread config to write there all defaults from all plugins
+        config = Config()
+        config.read_from_string(default_config)
+
+        # getting options' defaults from parser
+        defaults = dict(parser.parse_args([])._get_kwargs())
+        config.update_from_dict(defaults)
+
+        # now applying settings from the config file
+        if os.path.exists(config_filename):
+            config.read_from_file(config_filename)
+
+        # and finally, override them with command line options
+        # if there was specified value different from default
         args = parser.parse_args(command_line_args)
-        config.update_from_args(args)
+        args = dict(
+            (key, value)
+            for key, value in args._get_kwargs()
+                if value != defaults.get(key)
+        )
+        config.update_from_dict(args)
         self.config = config
 
         for key, value in config_dict.items():
@@ -382,25 +421,25 @@ class Bot(object):
             description='The Bot — Hubot\'s killer.'
         )
         parser.add_argument(
-            '--verbose', '-v', action='store_true', default=None,
+            '--verbose', '-v', action='store_true', default=False,
             help='Show more output. Default: False.'
         )
         parser.add_argument(
-            '--log-filename',
+            '--log-filename', default='thebot.log',
             help='Log\'s filename. Default: thebot.log.'
         )
         parser.add_argument(
-            '--storage-filename',
+            '--storage-filename', default='thebot.storage',
             help='Path to a database file, used for TheBot\'s memory. Default: thebot.storage.'
         )
 
         group = parser.add_argument_group('General options')
         group.add_argument(
-            '--adapters', '-a',
+            '--adapters', '-a', default='console',
             help='Adapters to use. You can specify a comma-separated list to use more than one adapter. Default: console.',
         )
         group.add_argument(
-            '--plugins', '-p',
+            '--plugins', '-p', default='image,math,todo',
             help='Plugins to use. You can specify a comma-separated list to use more than one plugin. Default: image.',
         )
         return parser
